/*******************************************************************************
 * $Id: AbstractMatch.cpp,v 1.5 2004/02/27 23:08:41 darling Exp $
 * This file is copyright 2002-2004 Aaron Darling.  All rights reserved.
 * Please see the file called COPYING for licensing, copying, and modification
 * rights.  Redistribution of this file, in whole or in part is prohibited
 * without express permission.
 ******************************************************************************/

#include "libMems/AbstractMatch.h"
#include "gn/gnException.h"
#include "gn/gnDebug.h"

AbstractMatch::AbstractMatch(){
	m_offset = 0;
	m_multiplicity = 0;
	m_firstStart = UINT32_MAX;
	m_seq_count = 0;
}

AbstractMatch::AbstractMatch( uint32 seq_count ){
	m_offset = 0;
	m_multiplicity = 0;
	m_firstStart = UINT32_MAX;
	m_seq_count = seq_count;
	memset( m_start, 0, sizeof( m_start ));
	memset( m_offsets, 0, sizeof( m_start ));
}


AbstractMatch::AbstractMatch(const AbstractMatch& mhe){
	(*this) = mhe;
}

AbstractMatch::~AbstractMatch(){

}

AbstractMatch& AbstractMatch::operator=(const AbstractMatch& mhe)
{
	m_offset = mhe.m_offset;
	m_seq_count = mhe.m_seq_count;
	memcpy( m_start, mhe.m_start, sizeof( m_start ) );
	memcpy( m_offsets, mhe.m_offsets, sizeof( m_offsets ) );
	m_multiplicity = mhe.m_multiplicity;
	m_firstStart = mhe.m_firstStart;
	m_matchnumber = mhe.m_matchnumber;
	return *this;
}

uint32 AbstractMatch::FirstStart() const{
	if(m_firstStart != UINT32_MAX)
		return m_firstStart;
	for(uint32 startI=0; startI < m_seq_count; startI++)
		if(m_start[startI] != MEM_NO_MATCH)
			return startI;
	return UINT32_MAX;
}


void AbstractMatch::CalculateOffset(){
	int64 ref_start = 0;
	uint32 seqI = 0;
	m_offset = 0;
//	m_matchnumber = valarray<bool>( false, seq_count );
	for( uint offsetI = 0; offsetI < SeqCount(); offsetI++ )
		m_offsets[ offsetI ] = 0;
	for(; seqI < m_seq_count; seqI++){
		if(m_start[seqI] != MEM_NO_MATCH){
			ref_start = m_start[seqI];
			m_multiplicity = 1;
			m_firstStart = seqI;
//			m_matchnumber[ seqI ] = true;
			break;
		}
	}
	m_matchnumber = 1;
	for(seqI++; seqI < SeqCount(); seqI++){
		m_matchnumber <<= 1;
		if(m_start[seqI] > 0){
			m_offsets[ seqI ] = m_start[seqI] - ref_start;
			m_offset += m_offsets[ seqI ];
			m_multiplicity++;
			m_matchnumber |= 1;
		}
		else if( m_start[seqI] < 0 ){
			m_offsets[ seqI ] = m_start[seqI] - (int64)Length( seqI ) - ref_start;
			m_offset += m_offsets[ seqI ];
			m_multiplicity++;
			m_matchnumber |= 1;
		}
	}
}
