/*******************************************************************************
 * $Id: Aligner.cpp,v 1.45 2004/02/27 23:08:41 darling Exp $
 * This file is copyright 2002-2004 Aaron Darling.  All rights reserved.
 * Please see the file called COPYING for licensing, copying, and modification
 * rights.  Redistribution of this file, in whole or in part is prohibited
 * without express permission.
 ******************************************************************************/

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWindows headers)
#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include "libMems/Aligner.h"
#include <map>
#include "libMems/MemSubsets.h"
#include <fstream>	// for debugging
#include <sstream>
#include "libMems/ClustalInterface.h"
#include "libMems/Islands.h"
#include <stack>
#include <algorithm>

class ScoreModel{
public:
	ScoreModel(){
		// initialize to default values
		gap_open = -6;
		gap_extend = -1;
		gap_close = -6;
		match = 1;
		mismatch = 1;
	}
	int gap_open;
	int gap_extend;
	int gap_close;
	int match;
	int mismatch;
};

int score( vector< AbstractMatch* >& matches, uint seq_1, uint seq_2 ){
	
	if( matches.size() == 0 )
		return 0;
//	uint seq_count = matches[ 0 ]->SeqCount();
	
//	ScoreModel sm;
	
	for( uint matchI = 0; matchI < matches.size(); matchI++ ){
//		matches[ matchI ]->Start( seq );
//		matches[ matchI ]->Score( sm, seq_1, seq_2 );
	}
	return 0;
}


boolean validateLCB( MatchList& lcb );
void validateRangeIntersections( vector< MatchList >& lcb_list  );
bool debug_shite = false;
const uint MIN_SEED_SIZE = 5;

boolean validateLCB( MatchList& lcb ){
	vector< Match* >::iterator lcb_iter = lcb.begin();
	if( lcb.size() == 0 )
		return true;
	uint seq_count = (*lcb_iter)->SeqCount();
	uint seqI = 0;
	boolean complain = false;
	for(; seqI < seq_count; seqI++ ){
		lcb_iter = lcb.begin();
		int64 prev_coord = 0;
		for(; lcb_iter != lcb.end(); lcb_iter++ ){
			if( (*lcb_iter)->Start( seqI ) == MEM_NO_MATCH )
				continue;
			else if( prev_coord != 0 && (*lcb_iter)->Start( seqI ) < prev_coord ){
				complain = true;
			}
			prev_coord = (*lcb_iter)->Start( seqI );
		}
	}
	return !complain;
}

void validateRangeIntersections( vector< MatchList >& lcb_list  ){
	if( lcb_list.size() == 0 )
		return;
	// build LCB ranges
	uint lcbI = 0;
	uint seqI = 0;
	vector< LCB > lcb_bounds;
	uint seq_count = lcb_list[ 0 ].seq_table.size();
	for( ; lcbI < lcb_list.size(); lcbI++ ){
		LCB cur_lcb;
		for( seqI = 0; seqI < seq_count; seqI++ ){
			cur_lcb.start.push_back( MEM_NO_MATCH );
			cur_lcb.length.push_back( 0 );
		}
		vector< Match* >::iterator match_iter = lcb_list[ lcbI ].begin();

		// find start in each sequence
		for(; match_iter != lcb_list[ lcbI ].end(); match_iter++ ){
			for( seqI = 0; seqI < seq_count; seqI++ ){
				if( cur_lcb.start[ seqI ] == MEM_NO_MATCH &&
					(*match_iter)->Start( seqI ) != MEM_NO_MATCH ){

					cur_lcb.start[ seqI ] = (*match_iter)->Start( seqI );
					if( cur_lcb.start[ seqI ] < 0 )
						cur_lcb.start[ seqI ] -= (*match_iter)->Length();
				}
			}
		}

		// find end in each sequence
		match_iter = lcb_list[ lcbI ].end();
		for(; match_iter != lcb_list[ lcbI ].begin(); match_iter-- ){
			vector< Match* >::iterator cur_iter = match_iter;
			cur_iter--;
			for( seqI = 0; seqI < seq_count; seqI++ ){
				if( cur_lcb.length[ seqI ] == 0 &&
					(*cur_iter)->Start( seqI ) != MEM_NO_MATCH ){

					cur_lcb.length[ seqI ] = (*cur_iter)->Start( seqI ) - cur_lcb.start[ seqI ];
					if( cur_lcb.start[ seqI ] < 0 )
						cur_lcb.length[ seqI ] -= (*cur_iter)->Length();
				}
			}
		}
		
		lcb_bounds.push_back( cur_lcb );
	}

	// just do a simple O(n^2) all against all bounds check
	uint lcbJ = 0;
	for( lcbI = 0; lcbI < lcb_bounds.size(); lcbI++ ){
		for( lcbJ = 0; lcbJ < lcb_bounds.size(); lcbJ++ ){
			if( lcbI == lcbJ )
				continue;
			for( seqI = 0; seqI < seq_count; seqI++ ){
				// intersection can be acceptable in some instances
				// containment is unacceptable
				// look for containment for now.
				if( lcb_bounds[ lcbI ].start[ seqI ] == 0 ||
					lcb_bounds[ lcbJ ].start[ seqI ] == 0 )
					continue;
				if( lcb_bounds[ lcbI ].start[ seqI ] < lcb_bounds[ lcbJ ].start[ seqI ] ){
					if( lcb_bounds[ lcbI ].start[ seqI ] + lcb_bounds[ lcbI ].length[ seqI ] >
						lcb_bounds[ lcbJ ].start[ seqI ] + lcb_bounds[ lcbJ ].length[ seqI ] ){
						
						// contained
						cerr << "seqI: " << seqI << '\t';
						cerr << lcb_bounds[ lcbI ].start[ seqI ] << ".." << lcb_bounds[ lcbI ].start[ seqI ] + lcb_bounds[ lcbI ].length[ seqI ];
						cerr << " and ";
						cerr << lcb_bounds[ lcbJ ].start[ seqI ] << ".." << lcb_bounds[ lcbJ ].start[ seqI ] + lcb_bounds[ lcbJ ].length[ seqI ];
						cerr << endl;

						cerr << "LCB " << lcbJ << " is contained by LCB " << lcbI;
						cerr << endl;
					}
				}
			}
		}
	}
};




/**
 * Delete overlapping regions in favor of the larger match.
 * This code isn't perfect, it can delete too many base pairs in some cases
 */
void Aligner::EliminateOverlaps( MatchList& ml ){
	if( ml.size() < 2 )
		return;
	vector< Match* > result_matches;
	SlotAllocator< Match >& static_match_allocator = SlotAllocator< Match >::GetSlotAllocator();
	for( uint seqI = 0; seqI < ml.seq_table.size(); seqI++ ){
		SingleStartComparator msc( seqI );
		sort( ml.begin(), ml.end(), msc );
		int64 matchI = 0;
		int64 nextI = 0;
		int64 deleted_count = 0;
		vector< Match* > new_matches;

		// scan forward to first defined match
		for(; matchI != ml.size(); matchI++ )
			if( ml[ matchI ]->Start( seqI ) != MEM_NO_MATCH )
				break;

		for(; matchI < ml.size(); matchI++ ){
			if( ml[ matchI ] == NULL )
				continue;
			
			for( nextI = matchI + 1; nextI < ml.size(); nextI++ ){
				if( ml[ nextI ] == NULL )
					continue;

				boolean deleted_matchI = false;
				// check for overlaps
				int64 startI = ml[ matchI ]->Start( seqI );
				int64 lenI = ml[ matchI ]->Length();
				int64 startJ = ml[ nextI ]->Start( seqI );
				int64 diff =  abs( startJ ) - abs( startI ) - lenI;

				if( diff < 0 ){
					diff = -diff;
					Match* new_match;
					// delete bases from the smaller match
//					if( ml[ nextI ]->Length() * ml[ nextI ]->Multiplicity() >= 
//						lenI * ml[ matchI ]->Multiplicity() ){
					if( ( ml[ nextI ]->Multiplicity() > ml[ matchI ]->Multiplicity() ) ||
 						( ml[ nextI ]->Multiplicity() == ml[ matchI ]->Multiplicity() && ml[ nextI ]->Length() > ml[ matchI ]->Length() ) ){
						// mem_iter is smaller
						new_match = ml[ matchI ]->Clone();
						// erase base pairs from new_match
						if( diff >= lenI ){
//							cerr << "Deleting " << **mem_iter << " at the hands of\n" << **next_iter << endl;
							static_match_allocator.Free( ml[ matchI ] );
							ml[ matchI ] = NULL;
							matchI--;
							deleted_matchI = true;
							deleted_count++;
						}else{
							if( startI > 0 ){
								ml[ matchI ]->CropEnd( diff );
								new_match->CropStart( new_match->Length() - diff );
							}else{
								ml[ matchI ]->CropStart( diff );
								new_match->CropEnd( new_match->Length() - diff );
							}
						}
					}else{
						// match_iter is smaller
						new_match = ml[ nextI ]->Clone();
						// erase base pairs from new_match
						if( diff >= ml[ nextI ]->Length() ){
//							cerr << "Deleting " << **next_iter << " at the hands of\n" << **mem_iter << endl;
							static_match_allocator.Free( ml[ nextI ] );
							ml[ nextI ] = NULL;
							deleted_count++;
						}else{
							if( startJ > 0 ){
								ml[ nextI ]->CropStart( diff );
								new_match->CropEnd( new_match->Length() - diff );
							}else{
								ml[ nextI ]->CropEnd( diff );
								new_match->CropStart( new_match->Length() - diff );
							}
						}

					}
					new_match->SetStart( seqI, 0 );
					new_match->CalculateOffset();
					if( new_match->Multiplicity() > 1 && new_match->Length() > 0 )
						new_matches.push_back( new_match );
					else
						static_match_allocator.Free( new_match );
					if( deleted_matchI )
						break;
				}else
					break;	// there are no more overlaps
			}
//			if( nextI > 1 )
//				cerr << "There were " << nextI << " overlaps\n";
//			if( nextI > config_value_2 )
//				__asm(nop);
		}

		if( deleted_count > 0 ){
			result_matches.reserve( ml.size() - deleted_count );
			for( int64 copyI = 0; copyI < ml.size(); copyI++ ){
				if( ml[ copyI ] != NULL )
					result_matches.push_back( ml[ copyI ] );
			}
			ml.clear();
			ml.insert( ml.end(), result_matches.begin(), result_matches.end() );
		}
		ml.insert( ml.end(), new_matches.begin(), new_matches.end() );
		result_matches.clear();
		new_matches.clear();
	}
		
}


const gnSeqI default_min_r_gap_size = 200;

Aligner::Aligner( uint seq_count ) :
match_allocator( SlotAllocator< Match >::GetSlotAllocator() )
{
	debug = false;
	this->seq_count = seq_count;
	min_recursive_gap_length = default_min_r_gap_size;
}

Aligner::Aligner( const Aligner& al ) : 
match_allocator( al.match_allocator )
{
	operator=( al );
}

Aligner& Aligner::operator=( const Aligner& al )
{
	ml = al.ml;
	gap_mh = al.gap_mh;
	nway_mh = al.nway_mh;
	seq_count = al.seq_count;
	debug = al.debug;
	
	LCB_minimum_density = al.LCB_minimum_density;
	LCB_minimum_range = al.LCB_minimum_range;
	
	cur_min_coverage = al.cur_min_coverage;
	min_recursive_gap_length = al.min_recursive_gap_length;

	return *this;
}

void Aligner::SetMinRecursionGapLength( gnSeqI min_r_gap ) {
	min_recursive_gap_length = min_r_gap;
}

/* returns true if all labels between start_label and end_label are contained in the no_match_labels set */
void Aligner::scanLabels( set< uint >& no_match_labels, uint& start_label, boolean forward ){
	uint labelI;
	// scan no_match_labels for consecutive labels starting at start_label until one is missing
	if( forward ){
		for( labelI = start_label + 1; ; labelI++){
			set< uint >::iterator  label_iter = no_match_labels.find( labelI );
			if( label_iter == no_match_labels.end() ){
				start_label = labelI - 1;
				break;
			}
		}
	}else{
		for( labelI = start_label; labelI > 0; labelI--){
			set< uint >::iterator  label_iter = no_match_labels.find( labelI - 1 );
			if( label_iter == no_match_labels.end() ){
				start_label = labelI;
				break;
			}
		}
	}
}

boolean Aligner::checkCollinearity( Match* m1, Match* m2 ){
	for( uint seqI = 0; seqI < seq_count; seqI++ ){
		if( m1->Start( seqI ) == MEM_NO_MATCH ||
			m2->Start( seqI ) == MEM_NO_MATCH )
			continue;
		if((( m1->Start( seqI ) > 0 &&
			m2->Start( seqI ) > 0 ) ||
			(m1->Start( seqI ) < 0 &&
			m2->Start( seqI ) < 0 )) &&
			m1->Start( seqI ) <= m2->Start( seqI ) )
			continue;
		return false;
	}
	return true;
}

void Aligner::scanFit( list< LabeledMem >& pair_list, list< LabeledMem >::iterator& list_iter, Match* new_match, uint sort_seq ){

	list< LabeledMem >::iterator cur_iter = list_iter;
	list< LabeledMem >::iterator last_iter = list_iter;
	int64 initial_start = abs( list_iter->mem->Start( sort_seq ) );

	uint match_count = 0;
	for(; last_iter != pair_list.end(); last_iter++ ){
		if( last_iter->mem->Start( sort_seq ) == MEM_NO_MATCH ){
			match_count++;
			continue;
		}
		if( abs( last_iter->mem->Start( sort_seq ) ) < initial_start ||
			abs( last_iter->mem->Start( sort_seq ) ) > new_match->Start( sort_seq ) )
			break;
		match_count++;
	}
	vector< vector< int > > score_vector;
	score_vector.reserve( seq_count - sort_seq - 1 );
	for( uint seqI = sort_seq + 1; seqI < seq_count; seqI++ ){
		vector< int > sv;
		score_vector.push_back( sv );
		score_vector[ score_vector.size() - 1 ].reserve( match_count );
	}
	uint matchI = 0;
	for(; cur_iter != last_iter; cur_iter++ ){
		
		for( uint seqI = sort_seq + 1; seqI < seq_count; seqI++ ){
			int64 p_start = cur_iter->mem->Start( seqI );
			int64 m_start = new_match->Start( seqI );
			p_start = p_start < 0 ? -p_start : p_start;
			m_start = m_start < 0 ? -m_start : m_start;
			if( m_start == MEM_NO_MATCH ){
				score_vector[ seqI - sort_seq - 1 ].push_back( 0 );
			}else if( p_start == MEM_NO_MATCH ){
				score_vector[ seqI - sort_seq - 1 ].push_back( 0 );
			}else if( p_start < m_start ){
				score_vector[ seqI - sort_seq - 1 ].push_back( 1 );
			}else
				score_vector[ seqI - sort_seq - 1 ].push_back( -1 );
		}
	}
	vector< int > scores;
	scores.reserve( match_count );
	for( matchI = match_count; matchI > 0; matchI-- )
		scores.push_back( 0 );
	for( uint seqI = 0; seqI < seq_count - sort_seq - 1; seqI++ ){
		boolean redefined = false;
		for( matchI = match_count; matchI > 0; matchI-- ){
			if( !redefined ){
				if( score_vector[ seqI ][ matchI - 1 ] >= 0 ){
					if( score_vector[ seqI ][ matchI - 1 ] == 1 )
						redefined = true;
					scores[ matchI - 1 ]++;
				}
			}else{
				if( score_vector[ seqI ][ matchI - 1 ] == -1 )
					redefined = false;
			}
		}
	}
	// find the first highest scoring match
	cur_iter = list_iter;
	int max_score = 0;
	for( matchI = 0; matchI < match_count; matchI++ ){
		if( scores[ matchI ] > max_score ){
			max_score = scores[ matchI ];
			list_iter = cur_iter;
		}
		cur_iter++;
	}
}

/**
 * Aaron's subset LCB algorithm.  
 */
void Aligner::AaronsLCB( MatchList& mlist, set<uint>& breakpoints ){
	if( mlist.size() == 0 )
		return;
	// can only look for breakpoints if there is more than one match!!
	if( mlist.size() == 1 ){
		breakpoints.insert( 0 );
		return;
	}

	SingleStartComparator msc( 0 );
	sort( mlist.begin(), mlist.end(), msc );
	vector<Match*>::iterator mem_iter = mlist.begin();
	list<LabeledMem> pair_list;
	
	map<uint, Match*> debug_label_map;
	boolean debugging = false;
	
	
	list< PlacementMatch > placement_list;
	
	for(; mem_iter != mlist.end(); mem_iter++ ){
		if( (*mem_iter)->Start( 0 ) != MEM_NO_MATCH ){		
			// add this one to the list.
			LabeledMem lm;
			lm.mem = *mem_iter;
			lm.label = 0;
			pair_list.push_back( lm );
		}else{
			PlacementMatch pm;
			pm.mem = *mem_iter;
			pm.iter = pair_list.end();
			placement_list.push_back( pm );
		}
	}
	LabeledMemComparator lmc( 0 );
	pair_list.sort( lmc );
	list< LabeledMem >::iterator pair_iter = pair_list.begin();
	for(; pair_iter != pair_list.end(); pair_iter++ ){
		PlacementMatch pm;
		pm.mem = pair_iter->mem;
		pm.iter = pair_iter;
		placement_list.push_back( pm );
	}
	
	// place all the subset matches from each sequence in the correct place in pair_list.
	for( uint seqI = 1; seqI < seq_count; seqI++ ){
		PlacementMatchComparator pmc( seqI );
		placement_list.sort( pmc );
		list< PlacementMatch >::iterator placement_prev;
		list< PlacementMatch >::iterator placement_iter = placement_list.begin();
		if( placement_iter->iter == pair_list.end() &&
			placement_iter->mem->Start( seqI ) != MEM_NO_MATCH ){
			LabeledMem lm;
			lm.mem = placement_iter->mem;
			lm.label = 0;
			pair_list.insert( pair_list.begin(), lm );
			placement_iter->iter = pair_list.begin();
		}

		for( placement_iter++; placement_iter != placement_list.end(); placement_iter++ ){
			placement_prev = placement_iter;
			placement_prev--;
			
			if( placement_iter->iter != pair_list.end() )
				continue;
			
			if( placement_iter->mem->Start( seqI ) == MEM_NO_MATCH )
				continue;
			
			list< LabeledMem >::iterator insert_iter = placement_prev->iter;
			if( insert_iter == pair_list.end() || placement_prev->mem->Start( seqI ) == MEM_NO_MATCH )
				insert_iter = pair_list.begin();
			else{
				if( insert_iter->mem->Start( seqI ) < 0 ){
					// invert if necessary and insert before
					if( placement_iter->mem->Start( seqI ) > 0 )
						placement_iter->mem->Invert();
					if( !checkCollinearity( placement_iter->mem, insert_iter->mem ) ){
						placement_iter->mem->Invert();
						scanFit( pair_list, insert_iter, placement_iter->mem, seqI );
						insert_iter++;
					}
				}else{
					// insert in the earliest place this match fits with surrounding matches
					scanFit( pair_list, insert_iter, placement_iter->mem, seqI );
					insert_iter++;
				}
			}
			
			LabeledMem lm;
			lm.mem = placement_iter->mem;
			lm.label = 0;
			pair_list.insert( insert_iter, lm );
			placement_iter->iter = insert_iter;
			placement_iter->iter--;
		}
	}
	boolean debug_labels = false;
	ofstream debug_label_file;
	if( debug_labels )
		debug_label_file.open( "label_debug.txt" );
	// number the LabeledMems in the pair_list
	uint cur_label = 0;
	mlist.clear();
	vector< LabeledMem > pair_vec;
	pair_vec.reserve( pair_list.size() );
	mlist.reserve( pair_list.size() );
	for( pair_iter = pair_list.begin(); pair_iter != pair_list.end(); pair_iter++ ){
		pair_iter->label = cur_label++;
		mlist.push_back( pair_iter->mem );
		pair_vec.push_back( *pair_iter );
		if( debug_labels ){
			debug_label_map.insert( map<uint, Match*>::value_type( pair_iter->label, pair_iter->mem ) );
			debug_label_file << pair_iter->label << '\t' << (*pair_iter->mem) << endl;
		}
	}
	if( debug_labels )
		debug_label_file.close();
	
	breakpoints.clear();
	pair_list.clear();
	vector< LabeledMem >::iterator pair_vec_iter;
	for( uint seqI = 1; seqI < seq_count; seqI++ ){
		// sort the list on the current genome
		LabeledMemComparator lmc( seqI );
		sort( pair_vec.begin(), pair_vec.end(), lmc );
		set< uint > no_match_labels;

		// debugging code
/*		stringstream debug_filename;
		debug_filename << "label_sort_" << seqI << ".txt";
		ofstream debug_file( debug_filename.str().c_str() );
		for( uint pairI = 0; pairI < pair_vec.size(); pairI++ ){
			debug_file << pair_vec[ pairI ].label << *pair_vec[ pairI ].mem << endl;
		}
		debug_file.close();
*/		// end debugging code
		
		pair_vec_iter = pair_vec.begin();
		uint block_start = pair_vec_iter->label;
		uint break_label = 0;
		for( pair_vec_iter++; pair_vec_iter != pair_vec.end(); pair_vec_iter++ ){
			vector<LabeledMem>::iterator pair_prev = pair_vec_iter;
			pair_prev--;
			break_label = 0;
			uint scan_label = 0;
			if( pair_prev->mem->Start( seqI ) == MEM_NO_MATCH ){
				no_match_labels.insert( set< uint >::value_type( pair_prev->label ) );
				// get the correct block start
				if( pair_vec_iter->mem->Start( seqI ) < 0 ){
					block_start = pair_vec_iter->label;
					scanLabels( no_match_labels, block_start, true );
				}else if( pair_vec_iter->mem->Start( seqI ) > 0 ){
					block_start = pair_vec_iter->label;
					scanLabels( no_match_labels, block_start, false );
				}
				
				continue;
			}

			if( pair_prev->mem->Start( seqI ) < 0 ){
				// this block would break at its start
				break_label = block_start;
			}else{
				// this block would break at its end
				break_label = pair_prev->label;
				scanLabels( no_match_labels, break_label, true );
			}
			if( pair_vec_iter->mem->Start( seqI ) < 0 ){
				// scan forward to the beginning of new block
				scan_label = pair_vec_iter->label;
				scanLabels( no_match_labels, scan_label, true );
			}else{
				// scan back to the beginning of new block
				scan_label = pair_vec_iter->label;
				scanLabels( no_match_labels, scan_label, false );
			}

			if( pair_vec_iter->mem->Start( seqI ) < 0 &&
				pair_prev->mem->Start( seqI ) < 0 ){
				if( scan_label + 1 == pair_prev->label )
					continue;
				if( debugging ){
					map< uint, Match* >::const_iterator debug_iter = debug_label_map.find( pair_vec_iter->label );
					while( debug_iter->first <= pair_prev->label ){
						cout << debug_iter->first << '\t' << *(debug_iter->second) << endl;
						debug_iter++;
					}
				}
			}else
			if( pair_vec_iter->mem->Start( seqI ) > 0 &&
				pair_prev->mem->Start( seqI ) > 0 ){
				
				if( scan_label - 1 == pair_prev->label )
					continue;
				if( debugging ){
					map< uint, Match* >::const_iterator debug_iter = debug_label_map.find( pair_prev->label );
					while( debug_iter->first <= pair_vec_iter->label ){
						cout << debug_iter->first << '\t' << *(debug_iter->second) << endl;
						debug_iter++;
					}
				}
			}
			// check if the missing matches are in the set of non-matches

			// since it didn't meet any of the above
			// criteria it's a breakpoint.  insert the label of the end of the current block
			// note that if it's a reverse complement block, the end label is really the start label
			breakpoints.insert( break_label );
			block_start = scan_label;
		}

		// insert the correct block ending
		if( pair_vec_iter != pair_vec.begin() ){
			pair_vec_iter--;
			
			if( pair_vec_iter->mem->Start( seqI ) < 0 ){
				break_label = block_start;
			}else{
				break_label = pair_vec_iter->label;
				scanLabels( no_match_labels, break_label, true );
			}
			breakpoints.insert( break_label );
		}
	}
}

void Aligner::GetLCBIdentityMatrix( MatchList& lcb, NumericMatrix< double >& identity, NumericMatrix< double >& range ){
	NumericMatrix< double > density( seq_count, seq_count );
	NumericMatrix< double > m_range( seq_count, seq_count );
	NumericMatrix< double > last_coord( seq_count, seq_count );
	uint seqI, seqJ;
	density.init( 0 );
	m_range.init( 0 );
	last_coord.init( 0 );

	vector< Match* >::iterator match_iter = lcb.begin();
	for( ; match_iter != lcb.end(); match_iter++ ){
		for( seqI = 0; seqI < seq_count; seqI++ ){
			if( (*match_iter)->Start( seqI ) == MEM_NO_MATCH )
				continue;
			for( seqJ = 0; seqJ < seq_count; seqJ++ ){
				if( (*match_iter)->Start( seqJ ) == MEM_NO_MATCH || seqI == seqJ )
					continue;
				last_coord( seqI, seqJ ) = (*match_iter)->End( seqI );
				density( seqI, seqJ ) += (*match_iter)->Length();
				if( m_range( seqI, seqJ ) == 0 ){
					m_range( seqI, seqJ ) = (*match_iter)->Start( seqI );
					if( m_range( seqI, seqJ ) < 0 )
						m_range( seqI, seqJ ) -= (*match_iter)->Length() - 1;
				}
			}
		}
	}
	for( seqI = 0; seqI < seq_count; seqI++ ){
		for( seqJ = 0; seqJ < seq_count; seqJ++ ){
			if( seqI == seqJ )
				density( seqI, seqJ ) = 1;
			else if( last_coord( seqI, seqJ ) != 0 ){
				m_range( seqI, seqJ ) = abs( last_coord( seqI, seqJ ) - m_range( seqI, seqJ ) ) + 1;
				density( seqI, seqJ ) = density( seqI, seqJ ) / (double)m_range( seqI, seqJ );
			}
		}
	}
	range = m_range;
	identity = density;
}

/**
 * Gets the LCB's density and range in each sequence
 */
void Aligner::GetLCBStats( MatchList& lcb, vector< double >& density, vector< int64 >& range ){
	// initialize density and range to 0
	density.clear();
	range.clear();
	density.reserve( seq_count );
	range.reserve( seq_count );
	vector< int64 > last_coord;
	uint seqI;
	for( seqI = 0; seqI < seq_count; seqI++ ){
		density.push_back( 0 );
		range.push_back( 0 );
		last_coord.push_back( 0 );
	}

	vector< Match* >::iterator match_iter = lcb.begin();
	for( ; match_iter != lcb.end(); match_iter++ ){
		for( seqI = 0; seqI < seq_count; seqI++ ){
			if( (*match_iter)->Start( seqI ) == MEM_NO_MATCH )
				continue;
			last_coord[ seqI ] = (*match_iter)->End( seqI );
			density[ seqI ] += (*match_iter)->Length();
			if( range[ seqI ] == 0 ){
				range[ seqI ] = (*match_iter)->Start( seqI );
				if( range[ seqI ] < 0 )
					range[ seqI ] -= (*match_iter)->Length() - 1;
			}
		}
	}
	for( seqI = 0; seqI < seq_count; seqI++ ){
		if( last_coord[ seqI ] != 0 ){
			range[ seqI ] = abs( last_coord[ seqI ] - range[ seqI ] ) + 1;
			density[ seqI ] = density[ seqI ] / (double)range[ seqI ];
		}
	}
}

void Aligner::GetLCBCoverage( MatchList& lcb, uint64& coverage ){
	vector< Match* >::iterator match_iter = lcb.begin();
	coverage = 0;
	for( ; match_iter != lcb.end(); match_iter++ ){
		coverage += (*match_iter)->Length() * (*match_iter)->Multiplicity();
	}
}
/**
 * Compares Matches.
 * Used by LCB construction algorithm 
 */
class LCBLeftComparator {
public:
	LCBLeftComparator( uint seq ){
		m_seq = seq;
	}
	LCBLeftComparator( LCBLeftComparator& lmc ){
		m_seq = lmc.m_seq;
	}
	boolean operator()(const LCB& a, const LCB& b) const{
		
		int64 a_start = a.left_end[ m_seq ], b_start = b.left_end[ m_seq ];
		if( a_start == MEM_NO_MATCH || b_start == MEM_NO_MATCH ){
			if( b_start != MEM_NO_MATCH )
				return true;
			return false;
		}
		if(a_start < 0)
			a_start = -a_start;
		if(b_start < 0)
			b_start = -b_start;

		int64 diff = a_start - b_start;
		return diff < 0;
	}
	~LCBLeftComparator() {
	}
protected:
	uint m_seq;
private:
	LCBLeftComparator();
};

class LCBIDComparator {
public:
//	LCBIDComparator(){}
	boolean operator()(const LCB& a, const LCB& b) const{
		return a.lcb_id < b.lcb_id;
	}
//	~LCBIDComparator() {
//	}
};

void Aligner::computeLCBAdjacencies( vector<MatchList>& lcb_list, vector< int64 >& weights, vector< LCB >& adjacencies ){
	IntervalList iv_list;
	for( uint lcbI = 0; lcbI < lcb_list.size(); lcbI++ ){
		Interval iv( seq_count );
		iv.matches.push_back( lcb_list[ lcbI ][ 0 ] );
		iv.matches.push_back( lcb_list[ lcbI ][ lcb_list[ lcbI ].size() - 1 ] );
		iv_list.push_back( iv );
	}
	computeLCBAdjacencies( iv_list, weights, adjacencies );
}

void Aligner::computeLCBAdjacencies_v2( vector<MatchList>& lcb_list, vector< int64 >& weights, vector< LCB >& adjacencies ){
	IntervalList iv_list;
	for( uint lcbI = 0; lcbI < lcb_list.size(); lcbI++ ){
		Interval iv( seq_count );
		iv.matches.push_back( lcb_list[ lcbI ][ 0 ] );
		iv.matches.push_back( lcb_list[ lcbI ][ lcb_list[ lcbI ].size() - 1 ] );
		iv_list.push_back( iv );
	}
	computeLCBAdjacencies_v2( iv_list, weights, adjacencies );
}

/**
 *  Redesign to be more intuitive.  left_adjacency is always left, regardless of LCB orientation
 */
void Aligner::computeLCBAdjacencies_v2( IntervalList& iv_list, vector< int64 >& weights, vector< LCB >& adjacencies ){
	if( iv_list.size() == 0 )
		return;	// there aren't any LCBs so there aren't any adjacencies!
	uint seqI;
	uint lcbI;
	for( lcbI = 0; lcbI < iv_list.size(); lcbI++ ){
		LCB lcb;
		for( seqI = 0; seqI < seq_count; seqI++ ){
			const Match* first_m = dynamic_cast< const Match* >( iv_list[ lcbI ].matches[ 0 ] );
			uint right_matchI = iv_list[ lcbI ].matches.size() - 1;
			const Match* last_m = dynamic_cast< const Match* >( iv_list[ lcbI ].matches[ right_matchI ] );

			int64 leftI = first_m->Start( seqI );
			if( leftI < 0 )
				leftI = last_m->Start( seqI );
			int64 rightI = 0;
			if( leftI > 0 ){
				rightI = last_m->Start( seqI ) + 
					  last_m->Length();
			}else{
				rightI = first_m->Start( seqI ) - first_m->Length();
			}
			lcb.left_end.push_back( leftI );
			lcb.right_end.push_back( rightI );
			lcb.left_adjacency.push_back( (uint)-1 );
			lcb.right_adjacency.push_back( (uint)-1 );

		}
		lcb.lcb_id = adjacencies.size();
		lcb.weight = weights[ lcbI ];
		lcb.recursed_left = false;
		lcb.recursed_right = false;
		adjacencies.push_back( lcb );
	}

	for( seqI = 0; seqI < seq_count; seqI++ ){
		LCBLeftComparator llc( seqI );
		sort( adjacencies.begin(), adjacencies.end(), llc );
		for( lcbI = 1; lcbI + 1 < iv_list.size(); lcbI++ ){
			adjacencies[ lcbI ].left_adjacency[ seqI ] = adjacencies[ lcbI - 1 ].lcb_id;
			adjacencies[ lcbI ].right_adjacency[ seqI ] = adjacencies[ lcbI + 1 ].lcb_id;
		}
		if( lcbI == iv_list.size() )
			lcbI--;	// need to decrement when there is only a single LCB

		// set first and last lcb adjacencies to -1
		adjacencies[ 0 ].left_adjacency[ seqI ] = (uint)-1;
		adjacencies[ lcbI ].right_adjacency[ seqI ] = (uint)-1;
		if( lcbI > 0 ){
			adjacencies[ 0 ].right_adjacency[ seqI ] = adjacencies[ 1 ].lcb_id;
			adjacencies[ lcbI ].left_adjacency[ seqI ] = adjacencies[ lcbI - 1 ].lcb_id;
		}
	}
	LCBIDComparator lic;
	sort( adjacencies.begin(), adjacencies.end(), lic );
	
}

/**
 *  Anybody have any idea how this code works?
 */
void Aligner::computeLCBAdjacencies( IntervalList& iv_list, vector< int64 >& weights, vector< LCB >& adjacencies ){
	uint seqI;
	uint lcbI;
	for( lcbI = 0; lcbI < iv_list.size(); lcbI++ ){
		LCB lcb;
		for( seqI = 0; seqI < seq_count; seqI++ ){
			const Match* first_m = dynamic_cast< const Match* >( iv_list[ lcbI ].matches[ 0 ] );
			uint right_matchI = iv_list[ lcbI ].matches.size() - 1;
			const Match* last_m = dynamic_cast< const Match* >( iv_list[ lcbI ].matches[ right_matchI ] );

			int64 leftI = first_m->Start( seqI );
			if( leftI < 0 )
				leftI = last_m->Start( seqI );
			lcb.left_end.push_back( leftI );
			int64 rightI = 0;
			if( leftI > 0 ){
				rightI = last_m->Start( seqI ) + 
					  last_m->Length();
			}else{
				rightI = first_m->Start( seqI ) - first_m->Length();
			}
			lcb.right_end.push_back( rightI );
			lcb.left_adjacency.push_back( (uint)-1 );
			lcb.right_adjacency.push_back( (uint)-1 );
		}
		lcb.lcb_id = adjacencies.size();
		lcb.weight = weights[ lcbI ];
		lcb.recursed_left = false;
		lcb.recursed_right = false;
		adjacencies.push_back( lcb );
	}

	for( seqI = 0; seqI < seq_count; seqI++ ){
		LCBLeftComparator llc( seqI );
		sort( adjacencies.begin(), adjacencies.end(), llc );
		for( lcbI = 1; lcbI + 1 < iv_list.size(); lcbI++ ){
			if( adjacencies[ lcbI ].left_end[ seqI ] > 0 ){
				adjacencies[ lcbI ].left_adjacency[ seqI ] = adjacencies[ lcbI - 1 ].lcb_id;
				adjacencies[ lcbI ].right_adjacency[ seqI ] = adjacencies[ lcbI + 1 ].lcb_id;
			}else{
				adjacencies[ lcbI ].left_adjacency[ seqI ] = adjacencies[ lcbI + 1 ].lcb_id;
				adjacencies[ lcbI ].right_adjacency[ seqI ] = adjacencies[ lcbI - 1 ].lcb_id;
			}
		}
		if( lcbI == iv_list.size() )
			lcbI--;	// need to decrement when there is only a single LCB

		// set first and last lcb adjacencies to -1
		if( adjacencies[ 0 ].left_end[ seqI ] > 0 ){
			adjacencies[ 0 ].left_adjacency[ seqI ] = (uint)-1;
			if( lcbI > 0 )
				adjacencies[ 0 ].right_adjacency[ seqI ] = adjacencies[ 1 ].lcb_id;
		}else{
			if( lcbI > 0 )
				adjacencies[ 0 ].left_adjacency[ seqI ] = adjacencies[ 1 ].lcb_id;
			adjacencies[ 0 ].right_adjacency[ seqI ] = (uint)-1;
		}

		if( adjacencies[ lcbI ].left_end[ seqI ] > 0 ){
			if( lcbI > 0 )
				adjacencies[ lcbI ].left_adjacency[ seqI ] = adjacencies[ lcbI - 1 ].lcb_id;
			adjacencies[ lcbI ].right_adjacency[ seqI ] = (uint)-1;
		}else{
			adjacencies[ lcbI ].left_adjacency[ seqI ] = (uint)-1;
			if( lcbI > 0 )
				adjacencies[ lcbI ].right_adjacency[ seqI ] = adjacencies[ lcbI - 1 ].lcb_id;
		}
	}
	LCBIDComparator lic;
	sort( adjacencies.begin(), adjacencies.end(), lic );
	
}

void Aligner::scanLeft( int& left_recurseI, vector< LCB >& adjacencies, int min_weight, int seqI ){
	while( left_recurseI != -1 && adjacencies[ left_recurseI ].weight < min_weight )
		left_recurseI = adjacencies[ left_recurseI ].left_adjacency[ seqI ];
}
void Aligner::scanRight( int& right_recurseI, vector< LCB >& adjacencies, int min_weight, int seqI ){
	while( right_recurseI != -1 && adjacencies[ right_recurseI ].weight < min_weight )
		right_recurseI = adjacencies[ right_recurseI ].right_adjacency[ seqI ];
}

void Aligner::recurseFromEnd( int recurseI, boolean going_right, vector< LCB >& adjacencies, int64 min_weight, MatchList& new_matches ){
	try{
	Match r_begin( seq_count, 0 );
	Match r_end( seq_count, 0 );
	MatchList recurse_matches;
	SubsetInclusionRemover sir;
	uint seqJ;
	recurse_matches.seq_table = new_matches.seq_table;
	recurse_matches.seq_filename = new_matches.seq_filename;
	for( seqJ = 0; seqJ < seq_count; seqJ++ ){
		// find the adjacent lcb to the left or right in each sequence
		int leftI = recurseI;
		int rightI = recurseI;
		if( going_right ){
			rightI = adjacencies[ rightI ].right_adjacency[ seqJ ];
			scanRight( rightI, adjacencies, min_weight, seqJ );
		}else{
			leftI = adjacencies[ leftI ].left_adjacency[ seqJ ];
			scanLeft( leftI, adjacencies, min_weight, seqJ );
		}
		
		int64 right_start;
		if( adjacencies[ recurseI ].left_end[ seqJ ] > 0 ){
			if( rightI != -1 )
				right_start = abs( adjacencies[ rightI ].left_end[ seqJ ] );
			else
				right_start = new_matches.seq_table[ seqJ ]->length();
		}else{
			if( rightI != -1 )
				right_start = -1 * (int64)abs( adjacencies[ rightI ].right_end[ seqJ ] );
			else
				right_start = -1;
		}
		
		int64 left_start;
		if( adjacencies[ recurseI ].left_end[ seqJ ] > 0 ){
			if( leftI != -1 )
				left_start = abs( adjacencies[ leftI ].right_end[ seqJ ] );
			else
				left_start = 1;
		}else{
			if( leftI != -1 )
				left_start = -1 * (int64)abs( adjacencies[ leftI ].left_end[ seqJ ] );
			else
				left_start = -1 * (int64)new_matches.seq_table[ seqJ ]->length();
		}

		r_begin.SetStart( seqJ, left_start );
		r_begin.SetLength( 1 );
		r_end.SetStart( seqJ, right_start );
		r_end.SetLength( 1 );
	}
	
	cout << "Recursing on: \n";
	cout << r_begin << endl;
	cout << r_end << endl;
	
	// find only n-way matches off the end of this thing
	Recursion( recurse_matches, &r_begin, &r_end, true );
	sir.EliminateLinkedInclusions( recurse_matches );
	new_matches.insert( new_matches.end(), recurse_matches.begin(), recurse_matches.end() );
	recurse_matches.clear();
	}catch( gnException& gne ){
		cerr << gne << endl;
	}catch( exception& e ){
		cerr << e.what() << endl;
	}catch(...){
		cerr << "When I say 'ohhh' you say 'shit'!\n";
	}
}

void Aligner::FillLCBGaps( vector< LCB >& adjacencies, int64 min_weight, MatchList& new_matches ){
	uint lcbI;
	uint seqI;
	
	for( lcbI = 0; lcbI < adjacencies.size(); lcbI++ ){
		if( adjacencies[ lcbI ].weight > min_weight )
			continue;
		for( seqI = 0; seqI < seq_count; seqI++ ){
			// scan left
			int left_recurseI = lcbI;
			scanLeft( left_recurseI, adjacencies, min_weight, seqI );
			if( adjacencies[ left_recurseI ].recursed_right != true ){
				// recurse to the right on this one
				recurseFromEnd( left_recurseI, true, adjacencies, min_weight, new_matches );
				adjacencies[ left_recurseI ].recursed_right = true;
			}

			// scan right
			int right_recurseI = lcbI;
			scanRight( right_recurseI, adjacencies, min_weight, seqI );
			if( adjacencies[ right_recurseI ].recursed_left != true ){
				// recurse to the left on this one
				recurseFromEnd( right_recurseI, false, adjacencies, min_weight, new_matches );
				adjacencies[ right_recurseI ].recursed_left = true;
			}
		}
	}
	
	EliminateOverlaps( new_matches );
	new_matches.MultiplicityFilter( seq_count );
}

void Aligner::SearchLCBGaps( vector< LCB >& adjacencies, MatchList& new_matches ) {
	if( adjacencies.size() == 0 )
		return;		// there aren't any intervening LCB regions!

	boolean debug_lcb_extension = false;	/**< enables debugging output */
	vector< vector< int64 > > iv_regions;	/**< lists of intervening regions between LCBs in each sequence */
											/**< start positions organized as iv_regions[ seqI ][ lcbI * 2 ] */
											/**< end positions organized as iv_regions[ seqI ][ lcbI * 2 + 1 ] */
	uint seqI = 0;
	int lcbI = 0;
	MatchList gap_list;
	gap_list.seq_table = vector< gnSequence* >( seq_count );	/**< intervening regions of sequences */
	gap_list.sml_table = vector< SortedMerList* >( seq_count );
	iv_regions = vector< vector< int64 > >( seq_count );

	// extract a gnSequence containing only the intervening regions
	for( seqI = 0; seqI < seq_count; seqI++ ){
		gap_list.seq_table[ seqI ] = new gnSequence();
		gap_list.sml_table[ seqI ] = new DNAMemorySML();

		// find the first LCB in this sequence
		for( lcbI = 0; lcbI < adjacencies.size(); lcbI++ ){
			if( adjacencies[ lcbI ].left_adjacency[ seqI ] == -1 )
				break;
		}
		// start concatenating the intervening regions
		// scan right
		int right_recurseI = lcbI;
		lcbI = -1;
		gnSeqI seq_len = 0;
		while( (lcbI != -1 || right_recurseI != -1 ) && right_recurseI < (int)adjacencies.size() ){
			int64 l_end = lcbI == -1 ? 1 : adjacencies[ lcbI ].right_end[ seqI ];
			int64 r_end = right_recurseI == -1 ? new_matches.seq_table[ seqI ]->length() : adjacencies[ right_recurseI ].left_end[ seqI ];

			l_end = abs( l_end );
			r_end = abs( r_end );
			if( l_end > r_end )
				l_end = r_end;
			
			lcbI = right_recurseI;
			if( right_recurseI != -1 )
				right_recurseI = adjacencies[ right_recurseI ].right_adjacency[ seqI ];
			try{
			if( debug_lcb_extension )
				cerr << "Adding " << seqI << "\t" << l_end << "\t" << r_end << "\t(" << r_end - l_end << " bp)" << endl;
			gap_list.seq_table[ seqI ]->append( new_matches.seq_table[ seqI ]->subseq( l_end, r_end - l_end ) );
			}catch(...){
				cout << "";
			}
			seq_len += r_end - l_end;
			iv_regions[ seqI ].push_back( l_end );
			iv_regions[ seqI ].push_back( r_end );
		}
		if( debug_lcb_extension )
			cerr << "seqI " << seqI << " seq_len: " << seq_len << endl;
	}
	//
	// search for MUMs in the intervening sequence regions
	//

	// calculate potential mer sizes for searches
	gnSeqI total_iv_length = 0;
	for( seqI = 0; seqI < seq_count; seqI++ ){
		total_iv_length += gap_list.seq_table[ seqI ]->length();
/*		cerr << "seqI: " << seqI << " length: " << gap_list.seq_table[ seqI ]->length();
		if( gap_list.seq_table[ seqI ]->isCircular() )
			cerr << " circular";
		cerr << "\n";
*/
	}
	total_iv_length /= seq_count;

	uint search_mer_size = (uint) (log( (long double)total_iv_length ) / log( (long double)2 ));
	if( total_iv_length == 0 )
		search_mer_size = 0;
	if( search_mer_size >= MIN_SEED_SIZE + 2 )
		search_mer_size -= 2;
	if( !(search_mer_size & 0x1 ) && search_mer_size >= MIN_SEED_SIZE )
		search_mer_size--;

	if( search_mer_size < MIN_SEED_SIZE )
		return;		// The seed size is too small to be significant
	
	//	Create sorted mer lists for the intervening gap region
	for( seqI = 0; seqI < seq_count; seqI++ ){
		gap_list.sml_table[ seqI ]->Clear();
		gap_list.sml_table[ seqI ]->Create( *(gap_list.seq_table[ seqI ]), search_mer_size );
	}

	//	Find all exact matches in the gap region
	nway_mh.Clear();
	nway_mh.FindMatches( gap_list );
	gap_list.MultiplicityFilter( seq_count );
//	nway_mh.GetMatchList( gap_list );
	
	if( debug_lcb_extension ){
		ofstream debug_extension_out( "new_extension_matches.txt" );
		gap_list.WriteList( debug_extension_out );
		debug_extension_out.close();
	}

	//
	// Transpose MUMs back into their original sequence coordinates
	//
	for( seqI = 0; seqI < seq_count; seqI++ )
		transposeMatches( gap_list, seqI, iv_regions[ seqI ] );

	EliminateOverlaps( gap_list );
	gap_list.MultiplicityFilter( seq_count );
	// filter out matches that are too short
	for( uint matchI = 0; matchI < gap_list.size(); matchI++ )
		if( gap_list[ matchI ]->Length() >= search_mer_size )
			new_matches.push_back( gap_list[ matchI ] );
}


class MatchLeftEndComparator {
public:
	MatchLeftEndComparator( unsigned seq = 0 ){
		m_seq = seq;
	}
	MatchLeftEndComparator( MatchLeftEndComparator& msc ){
		m_seq = msc.m_seq;
	}
	// TODO??  make this do a wraparound comparison if all is equal?
	boolean operator()(const AbstractMatch* a, const AbstractMatch* b) const{
		int32 start_diff = max( a->FirstStart(), m_seq ) - max( b->FirstStart(), m_seq );
		if(start_diff == 0){
			uint32 m_count = a->SeqCount();
			m_count = m_count <= b->SeqCount() ? m_count : b->SeqCount();
			for(uint32 seqI = m_seq; seqI < m_count; seqI++){
				int64 a_start = abs( a->Start( seqI ) ), b_start = abs( b->Start( seqI ) );
				int64 diff = a_start - b_start;
				if(a_start == (int64)MEM_NO_MATCH || b_start == (int64)MEM_NO_MATCH)
					continue;
				else if(diff == 0)
					continue;
				else
					return diff < 0;
			}
		}
		return start_diff < 0;
	}
private:
	unsigned m_seq;
};

/**
 * Transposes the coordinates of matches in mlist to correspond to the original
 * set of source sequence regions described by seq_regions, splitting matches if
 * necessary.
 */
void transposeMatches( MatchList& mlist, uint seqI, vector< int64 >& seq_regions ){
	uint matchI = 0;
	MatchLeftEndComparator msc( seqI );
	sort( mlist.begin(), mlist.end(), msc );
	uint regionI = 0;
	gnSeqI region_sum = seq_regions[ 1 ] - seq_regions[ 0 ];
	gnSeqI region_start_sum = 0;
	MatchList new_matches;

	for( ; matchI < mlist.size(); matchI++ ){
		// find the translated start coordinate for this match
		int64 trans_start = mlist[ matchI ]->Start( seqI );
		int64 iv_orig_start = trans_start;
		if( trans_start == 0 )
			continue;
		while( region_sum < abs( trans_start ) && regionI + 2 < seq_regions.size() ){
			regionI += 2;
			region_start_sum = region_sum;
			region_sum += seq_regions[ regionI + 1 ] - seq_regions[ regionI ];
		}

		if( trans_start < 0 )
			trans_start = -seq_regions[ regionI ] - ( -trans_start - region_start_sum ) + 1;
		else if( trans_start > 0 )
			trans_start = seq_regions[ regionI ] + ( trans_start - region_start_sum ) - 1;

		int64 trans_end = mlist[ matchI ]->Start( seqI );
		trans_end += trans_end > 0 ? mlist[ matchI ]->Length() - 1: -mlist[ matchI ]->Length() + 1;
		
		mlist[ matchI ]->SetStart( seqI, trans_start );
		
		// this bad boy may need to be split
		gnSeqI end_region_sum = region_sum;
		gnSeqI end_prev_sum = region_start_sum;
		uint end_regionI = regionI;
		Match* cur_match = mlist[ matchI ];
		while( end_region_sum < abs( trans_end ) && end_regionI + 2 < seq_regions.size() ){
			end_regionI += 2;

			Match* left_match = new Match( *cur_match );
			// clip off the part going to the other match
			if( left_match->Start( seqI ) < 0 ){
				cur_match->CropStart( abs( iv_orig_start ) + left_match->Length() - end_region_sum - 1);
				left_match->CropEnd( cur_match->Length() );
			}else{
				cur_match->CropEnd( abs( iv_orig_start ) + left_match->Length() - end_region_sum - 1);
				left_match->CropStart( cur_match->Length() );
			}

			iv_orig_start += iv_orig_start > 0 ? cur_match->Length(): -cur_match->Length();

			if( trans_start < 0 )
				trans_start = -seq_regions[ end_regionI ] - ( -iv_orig_start - end_region_sum ) + 1;
			else if( trans_start > 0 )
				trans_start = seq_regions[ end_regionI ] + ( iv_orig_start - end_region_sum ) - 1;
			
			left_match->SetStart( seqI, trans_start );

			cur_match = left_match;
			new_matches.push_back( left_match );

			end_prev_sum = end_region_sum;
			end_region_sum += seq_regions[ end_regionI + 1 ] - seq_regions[ end_regionI ];

		}
//		if( end_region_sum == abs( trans_end ) )
//			cerr << "Beware of a possible bug in transposeMatches()\n";
	}
	
	// voila... coordinates are translated
	mlist.insert( mlist.end(), new_matches.begin(), new_matches.end() );
}

void Aligner::ComputeLCBs( MatchList& meml, set<uint>& breakpoints, vector<MatchList>& lcb_list, vector<int64>& weights ){

	// there must be at least one end of a block defined
	if( breakpoints.size() < 1 )
		return;
	// organize the LCBs into different MatchList instances

	set<uint>::iterator break_iter = breakpoints.begin();
	uint prev_break = 0;	// prev_break is the first match in the current block
	MatchList lcb = meml;
	for( ; break_iter != breakpoints.end(); break_iter++ ){
		lcb.clear();
		lcb.insert( lcb.begin(), meml.begin() + prev_break, meml.begin() + *break_iter + 1 );
		prev_break = *break_iter + 1;
		
		// code to filter LCBs based on their coverage
		uint64 coverage;
		GetLCBCoverage( lcb, coverage );
		weights.push_back( coverage );
		if( coverage < cur_min_coverage || cur_min_coverage == -1 )
			cur_min_coverage = coverage;

		// add the new MatchList to the set if it made the cut
		lcb_list.push_back( lcb );
	}
}

void Aligner::FilterLCBs( MatchList& meml, set<uint>& breakpoints, vector<MatchList>& lcb_list, vector<Interval>& interval_list, double LCB_minimum_density, double LCB_minimum_range, boolean recursive ){
	// collect some stats about how things are being filtered in here:
	uint density_filter = 0;
	uint range_filter = 0;

	// there must be at least one end of a block defined
	if( breakpoints.size() < 1 )
		return;
	// organize the LCBs into different MatchList instances

	vector<Match*> match_list;
	match_list.reserve( meml.size() );
	match_list.insert( match_list.begin(), meml.begin(), meml.end() );
	meml.clear();
//	lcb_list.clear();
	set<uint>::iterator break_iter = breakpoints.begin();
	uint prev_break = 0;	// prev_break is the first match in the current block
	uint lcbI = 0;
	uint config_value = 0;
	for( ; break_iter != breakpoints.end(); break_iter++ ){
//		if( *break_iter - prev_break + 1 < LCB_minimum_matches ){
//			min_match_filter++;
//			prev_break = *break_iter + 1;
//			continue;
//		}
		// add the mems in this interval to a new MatchList
		lcbI++;
		MatchList lcb = meml;
		lcb.clear();
		lcb.insert( lcb.begin(), match_list.begin() + prev_break, match_list.begin() + *break_iter + 1 );
		prev_break = *break_iter + 1;
		
		// code to filter LCBs based on their coverage
		uint64 coverage;
		GetLCBCoverage( lcb, coverage );
		if( coverage <= LCB_minimum_range ){
			range_filter++;
			continue;
		}else if( (int64)coverage < cur_min_coverage || cur_min_coverage == -1 )
			cur_min_coverage = coverage;

/*		// code to filter LCBs based on their range
		vector< double > density;
		vector< int64 > range;
		GetLCBStats( lcb, density, range );
		uint best_seq = 0;
		double max_density = LCB_minimum_density;
		int64 max_range = LCB_minimum_range;
		uint seqI;
		for( seqI = 0; seqI < seq_count; seqI++ ){
			if( density[ seqI ] >= max_density &&
				range[ seqI ] >= max_range ){

				max_density = density[ seqI ];
				max_range = range[ seqI ];
				best_seq = seqI + 1;
			}
		}
		if( best_seq == 0 ){
			range_filter++;
			continue;
		}
*/
		// fill in the cracks between MUMs before calculating the density of coverage
//		if( recursive && lcb_list.size() >= 29 )
		Interval interval( seq_count );
		if( recursive )
			if( lcbI >= config_value )
				DoSomethingCool( lcb, interval );
		// add the new MatchList to the set if it made the cut
		lcb_list.push_back( lcb );
		interval_list.push_back( interval );
	}
	if( debug ){
		cout << "Of the " << breakpoints.size() << " possible LCBs,\n";
	//	cout << min_match_filter << " were filtered because they contained too few matches,\n";
		cout << density_filter << " were filtered because they weren't dense enough\n";
		cout << range_filter << " were filtered because their range was too little\n";
	}
}

void Aligner::Recursion( MatchList& r_list, Match* r_begin, Match* r_end, boolean nway_only ){
	try{
	gnSeqI gap_size = 0;
	uint seqI = 0;
//	gnSeqI min_gap_size = 0;
	boolean create_ok = true;
	// create gnSequences for each intervening region
	// create a MatchList for the intervening region
	MatchList gap_list;
	
	gap_list.seq_table.reserve( seq_count );
	gap_list.sml_table.reserve( seq_count );
	vector< int64 > starts;
	uint below_cutoff_count = 0;
// 
//	Get the sequence in the intervening gaps between these two matches
//
	for( seqI = 0; seqI < seq_count; seqI++ ){
		int64 gap_end = r_end->Start( seqI );
		int64 gap_start = r_begin->End( seqI ) + 1;
		if( r_end->Start( seqI ) == MEM_NO_MATCH ||
			r_begin->Start( seqI ) == MEM_NO_MATCH ){
			cerr << "It's screwed up\n";
			gap_list.seq_table.push_back( new gnSequence() );
			continue;
		}
		if( gap_end < 0 ){
			if ( gap_start > 0 ){
				create_ok = false;
				cerr << "It's screwed up\n";
				break; // bail out on directional inconsistency
			}
			gap_end = - r_begin->Start( seqI );
			gap_start = - r_end->Start( seqI ) + r_end->Length();
		}else if( gap_start <= 0 ){
			cerr << "It's screwed up\n";
			create_ok = false;
			break;	// bail out on directional inconsistency
		}
		int64 diff = gap_end - gap_start;
		diff = 0 < diff ? diff : 0;
		gap_size = diff < gap_size ? gap_size : diff;
//		if( seqI == 0 )
//			min_gap_size = diff;
//		min_gap_size = diff > min_gap_size ? min_gap_size : diff;

//		if( diff < 5 ){
//			create_ok = false;
//			break; // dont align less than 5 bp
//		}
		if( gap_start == 0 )
			cerr << "scheiss\n";
		if( debug )
			cout << r_list.seq_table[ seqI ]->length() << endl;
		if( diff < min_recursive_gap_length )
			below_cutoff_count++;
		starts.push_back( gap_start );
		gnSequence* new_seq = new gnSequence( r_list.seq_table[ seqI ]->subseq( gap_start, diff ) );
		gap_list.seq_table.push_back( new_seq );
		gap_list.sml_table.push_back( new DNAMemorySML() );
	}
	
	// only perform recursive anchoring if the gapped regions are long enough
	// otherwise just let ClustalW do the work
	if( below_cutoff_count + 1 < seq_count ){
		if( nway_only )
			nway_mh.Clear();
		else
			gap_mh.Clear();

		multimap< uint, uint > mer_sizes;
		// calculate potential mer sizes for searches
		for( seqI = 0; seqI < seq_count; seqI++ ){
			uint search_mer_size = (uint) (log( (long double)gap_list.seq_table[ seqI ]->length() ) / log( (long double)2 ));
			if( !(search_mer_size & 0x1 ) )
				search_mer_size++;
			if( gap_list.seq_table[ seqI ]->length() < 5 )
				search_mer_size = 0;
			mer_sizes.insert( multimap< uint, uint >::value_type( search_mer_size, seqI ) );
		}
		multimap< uint, uint >::iterator mer_iter = mer_sizes.end();
		mer_iter--;
		vector< uint > search_seqs;
		while( mer_iter != mer_sizes.end() ){
			uint prev_mer = mer_iter->first;
			uint new_seqs = 0;
			while( true ){
				if( mer_iter->first < MIN_SEED_SIZE )
					break;
				if( mer_iter->first == prev_mer || search_seqs.size() < 2 ){
					search_seqs.push_back( mer_iter->second );
					new_seqs++;
					if( mer_iter == mer_sizes.begin() ){
						mer_iter = mer_sizes.end();	// signify that the scan is complete
						break;
					}
					prev_mer = mer_iter->first;
					mer_iter--;
				}else
					break;
			}

			if( search_seqs.size() < 2 )
				break;
			// look for MUMs
			
			//
			//	Create sorted mer lists for the intervening gap region
			//

			for( uint seqI = 0; seqI < gap_list.seq_table.size(); seqI++ ){
				gap_list.sml_table[ seqI ]->Clear();
				gap_list.sml_table[ seqI ]->Create( *(gap_list.seq_table[ seqI ]), prev_mer );
			}
			//
			//	Find all exact matches in the gap region
			//
			MatchList cur_mems = gap_list;
			cur_mems.clear();
			if( nway_only ){
				// no sense in searching for matches in subsets!!
				if( search_seqs.size() < seq_count ){
					if( mer_iter->first < MIN_SEED_SIZE )
						break;
					continue;
				}
				nway_mh.ClearSequences();
				nway_mh.FindMatches( cur_mems );
			}else{
				gap_mh.ClearSequences();
				gap_mh.FindMatches( cur_mems );
			}
			if( mer_iter->first < MIN_SEED_SIZE )
				break;
		}
		if( nway_only )
			nway_mh.GetMatchList( gap_list );
		else
			gap_mh.GetMatchList( gap_list );
		
	//	SubsetInclusionRemover sir;
	//	sir.EliminateLinkedInclusions( gap_list );

		// delete overlaps/inclusions
	//	SubsetLinker slinker;
	//	slinker.Subsets( gap_list );
		
	//	SubsetInclusionRemover sir;
	//	sir.EliminateLinkedInclusions( gap_list );
		
	//	if( min_gap_size < search_mer_size )
	//		create_ok = false;
		if( gap_list.size() > 0 && create_ok ){

	/*		if( debug ){
				cout << "Starting mem: " << *r_begin << endl;
				cout << "Next mem: " << *r_end << endl;
				list<Match*>::iterator gappy_iter = gap_list.begin();
				while( gappy_iter != gap_list.end() ){
					cout << **gappy_iter;
					set<MatchID_t>& subsets = (*gappy_iter)->Subsets();
					set<MatchID_t>::iterator sub_iter = subsets.begin();
					cout << "\tsubsets: ";
					for(; sub_iter != subsets.end(); sub_iter++ ){
						cout << "\t" << *sub_iter;
					}
					set<MatchID_t>& supersets = (*gappy_iter)->Supersets();
					set<MatchID_t>::iterator super_iter = supersets.begin();
					cout << "\tsupersets: ";
					for(; super_iter != supersets.end(); super_iter++ ){
						cout << "\t" << *super_iter;
					}
					cout << endl;
					gappy_iter++;
				}
			}
	*/

			// move all the matches that were found
			vector< Match* >::iterator mum_iter = gap_list.begin();
			for( ; mum_iter != gap_list.end(); ){
				boolean add_ok = true;
				for( uint seqI = 0; seqI < (*mum_iter)->SeqCount(); seqI++ ){
					int64 gap_start;
					if( (*mum_iter)->Start( seqI ) == MEM_NO_MATCH )
						continue;
					else if( (*mum_iter)->Start( seqI ) < 0 ){
						gap_start = -r_begin->End( seqI );
						if( r_begin->End( seqI ) < 0 )
	//						gap_start = -r_end->Start( seqI ) + r_end->Length() - 1;
							gap_start = r_end->Start( seqI ) - r_end->Length() + 1;
						else
							add_ok = false;
						(*mum_iter)->SetStart( seqI, (*mum_iter)->Start( seqI ) + gap_start );
					}else{
						// insert them all before mem_iter
						gap_start = r_begin->End( seqI );
						if( gap_start < 0 ){
							gap_start = r_end->Start( seqI ) - r_end->Length() + 1;
							add_ok = false;
						}
						(*mum_iter)->SetStart( seqI, (*mum_iter)->Start( seqI ) + gap_start );
					}
				}
				if( add_ok )
					r_list.push_back( *mum_iter );
				else{
					(*mum_iter)->UnlinkSelf();
					match_allocator.Free( *mum_iter );
				}
				mum_iter++;
			}
	//		for( ; mum_iter != gap_list.end(); )
	//			match_allocator.Free( *mum_iter );
		}
	}
	// delete sequences and smls
	for( uint seqI = 0; seqI < gap_list.seq_table.size(); seqI++ )
		delete gap_list.seq_table[ seqI ];
	for( uint seqI = 0; seqI < gap_list.sml_table.size(); seqI++ )
		delete gap_list.sml_table[ seqI ];
		
	gap_list.seq_table.clear();
	gap_list.sml_table.clear();
	
	}catch( gnException& gne ){
		cerr << gne << endl;
	}catch( exception& e ){
		cerr << e.what() << endl;
	}catch(...){
		cerr << "When I say 'ohhh' you say 'shit'!\n";
	}
}

void Aligner::GetBestLCB( MatchList& r_list, MatchList& best_lcb ){
	if( debug )
		// now that matches are created, filter out all subset matches.
		cout << "Recursion starting with " << r_list.size() << " MUMs\n";
	
	// Remove linked inclusions
	SubsetInclusionRemover sir;
	sir.EliminateLinkedInclusions( r_list );
	EliminateOverlaps( r_list );

	if( debug )
		cout << "Removing linked inclusions yields " << r_list.size() << " MUMs\n";
	if( r_list.size() == 0 ){
		best_lcb.clear();
		return;
	}
	int lcb_count = -1;
	int prev_lcb_count;
	cur_min_coverage = -1;
	vector< MatchList > LCB_list;
	vector< int64 > weights;
	int64 min_coverage;
	do {
		min_coverage = cur_min_coverage;
		cur_min_coverage = -1;
		LCB_list.clear();
		weights.clear();

		// now do Aaron's version of the LCB algorithm
		set<uint> breakpoints;
		AaronsLCB( r_list, breakpoints );
		if( debug )
			cout << "The " << r_list.size() << " matches constitute " << breakpoints.size() << " breakpoints\n";

		// organize the LCBs into different MatchList instances
		ComputeLCBs( r_list, breakpoints, LCB_list, weights );
		r_list.clear();

		// copy the good LCBs back to r_list
		lcb_count = 0;
		for( uint lcbI = 0; lcbI < LCB_list.size(); lcbI++ ){
			if( weights[ lcbI ] > min_coverage ){
				r_list.insert( r_list.end(), LCB_list[ lcbI ].begin(), LCB_list[ lcbI ].end() );
				lcb_count++;
			}
		}
		
		if( lcb_count == 0 ){
			// take the one with the highest weight
			int64 max_weight = 0;
			int64 max_lcb = 0;
			for( uint lcbI = 0; lcbI < LCB_list.size(); lcbI++ ){
				if( weights[ lcbI ] >= max_weight ){

					max_weight = weights[ lcbI ];
					max_lcb = lcbI;
				}
			}
			r_list.insert( r_list.end(), LCB_list[ max_lcb ].begin(), LCB_list[ max_lcb ].end() );
			lcb_count = 1;
		}else{
			if( debug )
				cout << "There are " << lcb_count << " LCBs\n";
		}


//		if( cur_min_coverage < LCB_minimum_range )
//			min_coverage = cur_min_coverage;
//		cur_min_coverage = -1;
//		prev_lcb_count = lcb_count;


	}while( lcb_count != 1 );

	best_lcb = r_list;
}

void Aligner::DoSomethingCool( MatchList& mlist, Interval& iv ){
	boolean debug_recurse = false;
	int64 config_value = 138500;
	try{
	list< Match* > match_list;
	match_list.insert( match_list.end(), mlist.begin(), mlist.end() );
	mlist.clear();
	MatchList r_list = mlist;

	list< Match* >::iterator recurse_iter = match_list.begin();
	list< Match* >::iterator recurse_prev = match_list.begin();
	// scan ahead to the first n-way matches
	while( recurse_prev != match_list.end() && (*recurse_prev)->Multiplicity() != seq_count )
		recurse_prev++;

	recurse_iter = recurse_prev;
	if( recurse_iter != match_list.end() )
		recurse_iter++;
	while( recurse_iter != match_list.end() && (*recurse_iter)->Multiplicity() != seq_count )
		recurse_iter++;

	uint memI = 0;
	uint matchI = 0;
	while( recurse_iter != match_list.end() ){
		if( memI >= 50 && memI % 50 == 0 || debug)
			cout << "Number: " << memI << " match " << **recurse_iter << endl;
		memI++;
//		if( (*recurse_prev)->Start( 0 ) >= 10 )
//			debug_recurse = true;
		if( debug_recurse ){
			cout << "Recursing on " << endl;
			cout << **recurse_prev << " and " << endl;
			cout << **recurse_iter << endl;
		}
		
		if( (*recurse_prev)->Start( 0 ) == config_value )
			cout << "";
		
		// recurse on a pair of matches! 
		// this function should locate all matches between the two iterators
		// and add them to r_list		
		r_list.clear();
		ClustalResult* cr = NULL;
		boolean clustal_worked = false;
		MatchList best_lcb;
		Recursion( r_list, *recurse_prev, *recurse_iter );
		vector< Match* >::iterator r_iter = r_list.begin();
		if( debug_recurse ){
			cout << "Found matches " << endl;
			for(; r_iter != r_list.end(); r_iter++ )
				cout << **r_iter << endl;
		}
		uint nway_count = 0;
		if( r_list.size() > 0 ){
			// get the best LCB in this intervening region
			GetBestLCB( r_list, best_lcb );
			if( debug_recurse ){
				cout << "LCB matches " << endl;
				r_iter = best_lcb.begin();
				for(; r_iter != best_lcb.end(); r_iter++ )
					cout << **r_iter << endl;
			}
			for( matchI = 0; matchI < best_lcb.size(); matchI++ ){
				if( best_lcb[ matchI ]->Multiplicity() == seq_count )
					nway_count++;
			}
		}

		if( nway_count > 0 ){
			// replace the matches in the intervening region with the best LCB
			list< Match* >::iterator erase_begin = recurse_prev;
			erase_begin++;
			list< Match* >::iterator erase_iter = erase_begin;
			for(; erase_iter != recurse_iter; erase_iter++ )
				match_allocator.Free( *erase_iter );
			match_list.erase( erase_begin, recurse_iter );

			for( matchI = 0; matchI < best_lcb.size(); matchI++ ){
				if( best_lcb[ matchI ]->Multiplicity() == seq_count ){
					match_list.insert( recurse_iter, best_lcb[ matchI ] );
				}else
					match_allocator.Free( best_lcb[ matchI ] );
			}
		}else{
			// attempt a clustalW alignment
			cr = new ClustalResult();
			ClustalInterface& ci = ClustalInterface::getClustalInterface();
			clustal_worked = ci.ClustalAlign( *cr, *recurse_prev, *recurse_iter, r_list.seq_table );
/*			cout << **recurse_prev << endl;
			cout << **recurse_iter << endl;
			cout << cr;
			cout << endl;
*/		}
		if( nway_count == 0 && !clustal_worked && best_lcb.size() > 0 ){
			// Last resort:  add the subset matches
			list< Match* >::iterator erase_begin = recurse_prev;
			erase_begin++;
			list< Match* >::iterator erase_iter = erase_begin;
			for(; erase_iter != recurse_iter; erase_iter++ )
				delete *erase_iter;
			match_list.erase( erase_begin, recurse_iter );
			match_list.insert( recurse_iter, best_lcb.begin(), best_lcb.end() );
		}

		// scan ahead to the next pair of n-way matches
		list< Match* >::iterator recurse_next = recurse_prev;
		recurse_next++;
		while( (*recurse_next)->Multiplicity() != seq_count && recurse_next != recurse_iter)
			recurse_next++;
		if( recurse_next != recurse_iter )
			recurse_iter = recurse_next;
		else{
			// couldn't align anything else, add what was found to the Interval
			iv.add( *recurse_prev );
			if( clustal_worked )
				iv.add( cr );
			else if( best_lcb.size() > 0 )
				iv.matches.insert( iv.matches.end(), best_lcb.begin(), best_lcb.end() );
			recurse_prev = recurse_iter;
			recurse_iter++;
			while( recurse_iter != match_list.end() && (*recurse_iter)->Multiplicity() != seq_count )
				recurse_iter++;
		}
			
	}
	// get the last little bit at the end of the LCB.
	iv.matches.insert( iv.matches.end(), recurse_prev, recurse_iter );
	mlist.insert( mlist.end(), match_list.begin(), match_list.end() );
	iv.CalculateOffset();

	}catch( gnException& gne ){
		cerr << gne << endl;
	}catch(exception& e){
		cerr << e.what();
	}catch(...){
		cerr << "matrix exception?\n";
	}
}


void Aligner::align( MatchList& mlist, IntervalList& interval_list, double LCB_minimum_density, double LCB_minimum_range, boolean recursive, boolean extend_lcbs, string tree_filename ){
	seq_count = mlist.seq_table.size();
	this->LCB_minimum_density = LCB_minimum_density;
	this->LCB_minimum_range = LCB_minimum_range;
	
	// set the nway_mh mask
	uint64 nway_mask = 1;
	nway_mask <<= seq_count;
	nway_mask--;
	nway_mh.SetMask( nway_mask );
		
	// now that matches are created, filter out all subset matches.
	cout << "Starting with " << mlist.size() << " MUMs\n";
	
	// Remove linked inclusions
	SubsetInclusionRemover sir;
	sir.EliminateLinkedInclusions( mlist );
	cout << "ELI gives " << mlist.size() << " MUMs\n";
/*
	ofstream pre_overlap_debug( "pre_overlap_debug.txt" );
	if( pre_overlap_debug.is_open() )
		mlist.WriteList( pre_overlap_debug );
	pre_overlap_debug.close();
*/
	EliminateOverlaps( mlist );
	cout << "Eliminating overlaps yields " << mlist.size() << " MUMs\n";
/*	
	ofstream overlap_debug( "overlap_debug.txt" );
	if( overlap_debug.is_open() )
		mlist.WriteList( overlap_debug );
	overlap_debug.close();
*/	
	if( recursive || tree_filename != "" ){
		// Make a phylogenetic tree for ClustalW
		interval_list.seq_table = mlist.seq_table;
		interval_list.seq_filename = mlist.seq_filename;
		vector< pair< uint64, uint64 > > coverage_list;
		allocateDetailList( mlist, coverage_list );
		getLCBDetailList( mlist, coverage_list );
	//	PrintDetailList( seq_count, coverage_list, cout ); 
		NumericMatrix< double > distance;
		DistanceMatrix( seq_count, coverage_list, distance );
	//	distance.print(cout);
		ClustalInterface& ci = ClustalInterface::getClustalInterface();
		if( tree_filename == "" ){
			wxString temp_treename = wxGetTempFileName("guide_tree");
			tree_filename = temp_treename.c_str();
		}
		ci.SetDistanceMatrix( distance, tree_filename );
	}

	// Multiplicity Filter...
	mlist.MultiplicityFilter( seq_count );
	cout << "Multiplicity filter gives " << mlist.size() << " MUMs\n";

	if( mlist.size() == 0 )
		return;
		
	// do a first pass through computing the LCBs and extending their ends
	uint lcbI;
	int lcb_count = -1;
	set<uint> breakpoints;
	int64 min_coverage = -1;
	vector<MatchList> LCB_list;
	vector< int64 > weights;
	vector< LCB > adjacencies;
	MatchList new_matches;
	new_matches.seq_table = mlist.seq_table;
	new_matches.seq_filename = mlist.seq_filename;

	AaronsLCB( mlist, breakpoints );
	cout << "The " << mlist.size() << " matches constitute " << breakpoints.size() << " breakpoints\n";
	// organize the LCBs into different MatchList instances
	interval_list.clear();
	ComputeLCBs( mlist, breakpoints, LCB_list, weights );
//	FilterLCBs( mlist, breakpoints, LCB_list, interval_list, 0, -1, false );
	lcb_count = LCB_list.size();
	cout << "There are " << lcb_count << " LCBs\n";
/*	computeLCBAdjacencies( LCB_list, weights, adjacencies );
	for( lcbI = 0; lcbI < LCB_list.size(); lcbI++ ){
		recurseFromEnd( lcbI, true, adjacencies, -1, new_matches );
		recurseFromEnd( lcbI, false, adjacencies, -1, new_matches );
	}
	EliminateOverlaps( new_matches );
	new_matches.MultiplicityFilter( seq_count );
	mlist.insert( mlist.end(), new_matches.begin(), new_matches.end() );
*/	new_matches.clear();
	LCB_list.clear();
	
	// now get to the real meat and potatoes.  i'm hungry.  mmmmm, DNA.
	lcbI = 0;
	lcb_count = -1;
	int prev_lcb_count;
	double min_density = 0;
	double min_range = 5;
	boolean do_recursion = false;
	cur_min_coverage = -1;
	min_coverage = -1;
	do {
		if( cur_min_coverage < LCB_minimum_range )
			min_coverage = cur_min_coverage;
		cur_min_coverage = -1;
		prev_lcb_count = lcb_count;

		// now do Aaron's version of the LCB algorithm
		adjacencies.clear();
		breakpoints.clear();
		weights.clear();
		LCB_list.clear();
		new_matches.clear();
		AaronsLCB( mlist, breakpoints );
		// organize the LCBs into different MatchList instances
		ComputeLCBs( mlist, breakpoints, LCB_list, weights );
		mlist.clear();
//		computeLCBAdjacencies( LCB_list, weights, adjacencies );
//		FillLCBGaps( adjacencies, min_coverage, new_matches );
//		EliminateOverlaps( new_matches );
//		new_matches.MultiplicityFilter( seq_count );
		if( new_matches.size() > 0 ){
			// need to stabilize the number of LCBs at the current coverage weight
			// cur_min_coverage must be monotonically increasing
			mlist.insert( mlist.end(), new_matches.begin(), new_matches.end() );
			cur_min_coverage = cur_min_coverage <= min_coverage ? min_coverage + 1 : cur_min_coverage;
		}
		// copy the good LCBs back to mlist
		lcb_count = 0;
		for( lcbI = 0; lcbI < LCB_list.size(); lcbI++ ){
			if( weights[ lcbI ] > min_coverage ){
				mlist.insert( mlist.end(), LCB_list[ lcbI ].begin(), LCB_list[ lcbI ].end() );
				lcb_count++;
			}
		}

		cout << "The " << mlist.size() << " matches constitute " << lcb_count << " LCBs covering at least " << min_coverage << "b.p.\n";
		if( lcb_count == 0 )
			break;
	}while( (cur_min_coverage < LCB_minimum_range) || ( lcb_count != prev_lcb_count ) );
	
	
	int64 total_weight = 0;
	int64 prev_total_weight = 0;
	uint weightI = 0;
	boolean debug_lcb_extension = false;
	for( ; weightI < weights.size(); weightI++ )
		if( weights[ weightI ] > min_coverage )
			total_weight += weights[ weightI ];

	if( extend_lcbs ){
		cout << "Performing LCB extension\n";
		do {
			// search the gaps between the LCBs to extend the ends of LCBs
			adjacencies.clear();
			breakpoints.clear();
			weights.clear();
			LCB_list.clear();
			new_matches.clear();
			AaronsLCB( mlist, breakpoints );
			// organize the LCBs into different MatchList instances
			ComputeLCBs( mlist, breakpoints, LCB_list, weights );
// begin debug code
			if( debug_lcb_extension ){
				ofstream debug_extension_out( "extension_lcbs.txt" );
				for( uint lcbI = 0; lcbI < LCB_list.size(); lcbI++ )
					LCB_list[ lcbI ].WriteList( debug_extension_out );
				debug_extension_out.close();
			}
// end debug code
			computeLCBAdjacencies_v2( LCB_list, weights, adjacencies );
			SearchLCBGaps( adjacencies, new_matches );
// begin debug code
			if( debug_lcb_extension ){
				ofstream debug_newmatches_out( "new_matches.txt" );
				new_matches.WriteList( debug_newmatches_out );
				debug_newmatches_out.close();
			}
// end debug code			
			mlist.insert( mlist.end(), new_matches.begin(), new_matches.end() );

			// force the set of Matches to stabilize into LCBs again
			min_coverage = -1;
			do {
				if( cur_min_coverage < LCB_minimum_range )
					min_coverage = cur_min_coverage;
				cur_min_coverage = -1;
				prev_lcb_count = lcb_count;

				// now do Aaron's version of the LCB algorithm
				adjacencies.clear();
				breakpoints.clear();
				weights.clear();
				LCB_list.clear();
				AaronsLCB( mlist, breakpoints );
				// organize the LCBs into different MatchList instances
				ComputeLCBs( mlist, breakpoints, LCB_list, weights );
				mlist.clear();
				// copy the good LCBs back to mlist
				lcb_count = 0;
				for( lcbI = 0; lcbI < LCB_list.size(); lcbI++ ){
					if( weights[ lcbI ] > min_coverage ){
						mlist.insert( mlist.end(), LCB_list[ lcbI ].begin(), LCB_list[ lcbI ].end() );
						lcb_count++;
					}
				}

				cout << "The " << mlist.size() << " matches constitute " << lcb_count << " LCBs covering at least " << min_coverage << "b.p.\n";
				if( lcb_count == 0 )
					break;
			}while( (cur_min_coverage < LCB_minimum_range) || ( lcb_count != prev_lcb_count ) );
			
			// calculate the new total LCB weight
			prev_total_weight = total_weight;
			total_weight = 0;
			for( weightI = 0; weightI < weights.size(); weightI++ )
				total_weight += weights[ weightI ];
			cout << "Previous weight: " << prev_total_weight << " new weight: " << total_weight << endl;
			if( prev_total_weight > total_weight )
				cerr << "Error! Previous weight: " << prev_total_weight << " new weight: " << total_weight << endl;
		}while( total_weight > prev_total_weight );
	}

	// need to fill in the interval list
	for( lcbI = 0; lcbI < LCB_list.size(); lcbI++ ){
		Interval new_iv( seq_count );
		interval_list.push_back( new_iv );
		Interval& iv = interval_list[ interval_list.size() - 1 ];
		if( !recursive ){
			iv.matches.insert( iv.matches.begin(), LCB_list[ lcbI ].begin(), LCB_list[ lcbI ].end() );
		}else{
			DoSomethingCool( LCB_list[ lcbI ], iv );
		}
	}
}

void Aligner::readLCBlist( vector< MatchList >& LCB_list, istream& lcb_in, const MatchList& model_lcb ){
	string tag;
	gnSeqI len;
	int64 start;
	Match mhe;
	unsigned int seq_count;
	vector<Match*> match_list;
	MatchList new_lcb = model_lcb;
	new_lcb.clear();
	
	lcb_in >> tag;	//format version tag
	if( tag != "FormatVersion" ){
		Throw_gnEx(InvalidFileFormat());
	}
	lcb_in >> tag;	//format version
	if( tag != "2" ){
		Throw_gnEx(InvalidFileFormat());
	}
	lcb_in >> tag;	//sequence count tag
	if( tag != "SequenceCount" ){
		Throw_gnEx(InvalidFileFormat());
	}
	lcb_in >> seq_count;	//sequence count
	if(seq_count < 2){
		Throw_gnEx(InvalidFileFormat());
	}

	mhe = Match(seq_count, 23, Match::extended);
	
	// read the sequence file names and lengths
	for( unsigned int seqI = 0; seqI < seq_count; seqI++ ){
		lcb_in >> tag;	// name tag
		lcb_in >> tag;	// name
//		new_lcb.seq_filename.push_back(tag);
//		try{
//			gnSequence *new_seq = new gnSequence();
//			new_seq->LoadSource(tag);
//			seq_table.push_back( new_seq );
//		}catch( gnException& gne );
		lcb_in >> tag;	// length tag
		lcb_in >> tag;	// length
	}
	// read the number of matches
	unsigned int match_count;
	lcb_in >> tag;	// match count tag
	lcb_in >> match_count;	// match count
	
	int lcb_id = 0;
	int prev_lcb_id = -1;
	while(true){
		lcb_in >> lcb_id;
		if( lcb_id != prev_lcb_id && prev_lcb_id != -1){
			// add the new lcb.
			LCB_list.push_back( new_lcb );
			LCB_list[ LCB_list.size() - 1 ].insert( LCB_list[ LCB_list.size() - 1 ].begin(), match_list.begin(), match_list.end() );
			match_list.clear();
		}
		prev_lcb_id = lcb_id;
		
		lcb_in >> len;
		if(!lcb_in.good() || lcb_in.eof() )
			break;
		mhe.SetLength(len);
		for(uint32 seqI = 0; seqI < seq_count; seqI++){
			lcb_in >> start;
			mhe.SetStart(seqI, start);
		}
		//break if the stream ended
		if(!lcb_in.good() || lcb_in.eof() )
			break;
		mhe.CalculateOffset();
		match_list.push_back( new Match(mhe) );
	}
	// add the last lcb.
	LCB_list.push_back( new_lcb );
	LCB_list[ LCB_list.size() - 1 ].insert( LCB_list[ LCB_list.size() - 1 ].begin(), match_list.begin(), match_list.end() );

	uint total_matches = 0;
	for( uint lcbI = 0; lcbI < LCB_list.size(); lcbI++ ){
		total_matches += LCB_list[ lcbI ].size();
	}
	if( match_count != total_matches ){
		Throw_gnEx(InvalidFileFormat());
	}

}

void Aligner::writeLCBlist( const vector<MatchList>& LCB_list, ostream& lcb_out ){
	if( LCB_list.size() == 0 )
		return;
	
	const vector<Match*>& match_list = LCB_list[0];
	
	Match* first_mem = *(match_list.begin());
	unsigned int seq_count = first_mem->SeqCount();

	lcb_out << "FormatVersion" << '\t' << 2 << "\n";
	lcb_out << "SequenceCount" << '\t' << seq_count << "\n";
	for(unsigned int seqI = 0; seqI < seq_count; seqI++){
		lcb_out << "Sequence" << seqI << "File" << '\t';
		if( LCB_list[0].seq_filename.size() > seqI )
			lcb_out << LCB_list[0].seq_filename[seqI];
		else
			lcb_out << "null";
		lcb_out << "\n";
		lcb_out << "Sequence" << seqI << "Length" << '\t';
		if( LCB_list[0].seq_table.size() > seqI )
			lcb_out << LCB_list[0].seq_table[seqI]->length();
		else
			lcb_out << "0";
		lcb_out << "\n";
	}

	uint match_count = 0;
	uint lcbI;
	for( lcbI = 0; lcbI < LCB_list.size(); lcbI++ ){
		match_count += LCB_list[ lcbI ].size();
	}
	lcb_out << "MatchCount" << '\t' << match_count << endl;

	for( lcbI = 0; lcbI < LCB_list.size(); lcbI++ ){
		//get all the mems out of the lists and write them out
	    vector<Match*>::const_iterator mem_iter = LCB_list[ lcbI ].begin();
		for(; mem_iter != LCB_list[ lcbI ].end(); mem_iter++)
			lcb_out << lcbI << '\t' << **mem_iter << "\n";
	}
}

void Aligner::writeLCBstats( vector<MatchList>& LCB_list, ostream& os ){
	if( LCB_list.size() == 0 )
		return;
	if( LCB_list[0].size() == 0 )
		return;
	seq_count = (*LCB_list[0].begin())->SeqCount();

	NumericMatrix< double > total_identity( seq_count, seq_count );
	NumericMatrix< double > total_range( seq_count, seq_count );
	total_range.init( 0 );
	total_identity.init( 0 );
	for( uint lcbI = 0; lcbI < LCB_list.size(); lcbI++ ){
		NumericMatrix<double> identity;
		NumericMatrix<double> range;
//		vector< int64 > range;
		vector< double > density;
		
//		GetLCBStats( LCB_list[ lcbI ], density, range );
		GetLCBIdentityMatrix( LCB_list[ lcbI ], identity, range );
		for( uint colI = 0; colI < seq_count; colI++ ){
			for( uint rowI = 0; rowI < seq_count; rowI++ ){
				total_range( rowI, colI ) += range( rowI, colI );
				total_identity( rowI, colI ) += range( rowI, colI ) * identity( rowI, colI );
			}
		}
		os << lcbI << '\t' << LCB_list[ lcbI ].size() << endl;
		os << "LCB " << lcbI << endl;
		os << "Range:\n";
		range.print( os );
		os << endl;
		os << "Identity:\n";
		identity.print( os );
		os << endl << endl;
	}
	for( uint colI = 0; colI < seq_count; colI++ ){
		for( uint rowI = 0; rowI < seq_count; rowI++ ){
			total_identity( rowI, colI ) /= total_range( rowI, colI );
		}
	}
	os << "Total identity:\n";
	total_identity.print( os );
}
