/*******************************************************************************
 * $Id: Interval.h,v 1.3 2004/02/27 23:08:55 darling Exp $
 * This file is copyright 2002-2004 Aaron Darling.  All rights reserved.
 * Please see the file called COPYING for licensing, copying, and modification
 * rights.  Redistribution of this file, in whole or in part is prohibited
 * without express permission.
 ******************************************************************************/

#ifndef _Interval_h_
#define _Interval_h_

#include "gnClone.h"
#include "libMems/AbstractMatch.h"
#include "libMems/ClustalInterface.h"
#include "libMems/gnAlignedSequences.h"
#include <iostream>
#include <vector>

class Interval : public AbstractMatch {
public:
//	Interval(){};
	Interval( uint seq_count, gnSeqI align_length = 0 );
	Interval( const Interval& cr );
	~Interval(){};
	Interval& operator=( const Interval& cr );
	
	Interval* Clone() const;
	
	/**
	 * Writes this Interval to the specified output stream (e.g. cout).
	 */
	friend std::ostream& operator<<(std::ostream& os, const Interval& cr); //write to source.

	/**
	 * Reads a Interval from the specified input stream (e.g. cin).
	 */
	friend std::istream& operator>>(std::istream& is, const Interval& cr); //read from source

	// Inherited methods from AbstractMatch:
	virtual gnSeqI Length( uint seqI ) const { return length[ seqI ]; }
	virtual void SetLength( gnSeqI len, uint seqI ){};
	virtual int64 End(uint32 endI) const;
	virtual void Invert();
	virtual int64 GapSize(const AbstractMatch& mhe, uint32 seqI);
	virtual void CropStart(gnSeqI crop_amount);
	virtual void CropEnd(gnSeqI crop_amount);

	virtual void CalculateOffset();

	void add( AbstractMatch* am ){ matches.push_back( am ); }
	uint score( uint seqI, uint seqJ ){return 0;};
	
	/** 
	 * Get a gnAlignedSequences object
	 */
	virtual void GetAlignedSequences( gnAlignedSequences& gnas, const vector< gnSequence* >& seq_table ) const;

	vector< AbstractMatch* > matches;

protected:
	void update();
	NumericMatrix< double > identity;
	gnSeqI align_length;
	vector< int64 > length;
};

inline
Interval::Interval( uint seq_count, gnSeqI align_length ) : AbstractMatch( seq_count ){
	this->align_length = align_length;
	for( uint seqI = 0; seqI < seq_count; seqI++ ){
		length.push_back( 0 );
	}
	identity = NumericMatrix< double >( seq_count, seq_count );
}

inline
Interval::Interval( const Interval& cr ){
	*this = cr;
}

inline
Interval& Interval::operator=( const Interval& cr ){
	AbstractMatch::operator=( cr );
	matches = cr.matches;
	identity = cr.identity;
	align_length = cr.align_length;
	length = cr.length;
	return *this;
}

inline
Interval* Interval::Clone() const {
	return new Interval( *this ); 

}

#endif	// _Interval_h_
