/*******************************************************************************
 * $Id: MaskedMemHash.cpp,v 1.2 2004/02/27 23:08:41 darling Exp $
 * This file is copyright 2002-2004 Aaron Darling.  All rights reserved.
 * Please see the file called COPYING for licensing, copying, and modification
 * rights.  Redistribution of this file, in whole or in part is prohibited
 * without express permission.
 ******************************************************************************/

#include "libMems/MaskedMemHash.h"
#include <list>

MaskedMemHash::MaskedMemHash(){
	seq_mask = 0;
}


MaskedMemHash::MaskedMemHash(const MaskedMemHash& mh) : MemHash(mh){
	*this = mh;
}

MaskedMemHash& MaskedMemHash::operator=( const MaskedMemHash& mh ){
	seq_mask = mh.seq_mask;
	return *this;
}

MaskedMemHash* MaskedMemHash::Clone() const{
	return new MaskedMemHash(*this);
}

boolean MaskedMemHash::HashMatch(list<idmer>& match_list){
	//check that there is at least one forward component
	match_list.sort(&idmer_id_lessthan);
	// initialize the hash entry
	Match mhe = Match(seq_count, mer_size);
	mhe.SetLength(mer_size);
	
	//Fill in the new Match and set direction parity if needed.
	list<idmer>::iterator iter = match_list.begin();
	for(; iter != match_list.end(); iter++)
		mhe.SetStart(iter->id, iter->position + 1);
	SetDirection(mhe);
	mhe.CalculateOffset();
	if( mhe.MatchNumber() == seq_mask )
		AddHashEntry(mhe);

	return true;
}

